Package: ncmeta
Title: Straightforward 'NetCDF' Metadata
Version: 0.5.0
Authors@R: c(person("Michael", "Sumner", email = "mdsumner@gmail.com", role = c("aut", "cre", "cph")), 
             person("Tomas", "Remenyi", role = "ctb"), 
             person("Ben", "Raymond", role = "ctb"), 
             person("David", "Blodgett", role = "ctb"), 
             person("Milton", "Woods", role = "ctb"), 
             person("Patrick", "Van Laake", role = "ctb"))
Description: Extract metadata from 'NetCDF' data sources, these can be files, file handles or
 servers. This package leverages and extends the lower level functions of the 'RNetCDF' package 
 providing a consistent set of functions that all return data frames. We introduce named concepts 
 of 'grid', 'axis' and 'source' which are all meaningful entities without formal definition in the 
 'NetCDF' library <https://www.unidata.ucar.edu/software/netcdf>. 'RNetCDF' matches the library 
 itself with only the named concepts of 'variables', 'dimensions' and 'attributes'. 
Depends: R (>= 3.3.0)
License: GPL-3
Encoding: UTF-8
Imports: dplyr, rlang, RNetCDF, tibble, stats, tidyr (>= 1.0.0), CFtime
Suggests: testthat
URL: https://github.com/hypertidy/ncmeta,
        https://hypertidy.github.io/ncmeta/
BugReports: https://github.com/hypertidy/ncmeta/issues
ByteCompile: TRUE
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-29 06:30:22 UTC; mdsumner
Author: Michael Sumner [aut, cre, cph],
  Tomas Remenyi [ctb],
  Ben Raymond [ctb],
  David Blodgett [ctb],
  Milton Woods [ctb],
  Patrick Van Laake [ctb]
Maintainer: Michael Sumner <mdsumner@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-29 17:10:02 UTC
