# Generated by roxygen2: do not edit by hand

S3method(nc_att,NetCDF)
S3method(nc_att,character)
S3method(nc_atts,NetCDF)
S3method(nc_atts,character)
S3method(nc_axes,NetCDF)
S3method(nc_axes,character)
S3method(nc_axis,NetCDF)
S3method(nc_axis,character)
S3method(nc_coord_var,NetCDF)
S3method(nc_coord_var,character)
S3method(nc_dim,NetCDF)
S3method(nc_dim,character)
S3method(nc_dim,ncdf4)
S3method(nc_dims,NetCDF)
S3method(nc_dims,character)
S3method(nc_dims,ncdf4)
S3method(nc_extended,NetCDF)
S3method(nc_extended,character)
S3method(nc_extended,ncdf4)
S3method(nc_gm_to_prj,data.frame)
S3method(nc_gm_to_prj,list)
S3method(nc_grid_mapping_atts,NetCDF)
S3method(nc_grid_mapping_atts,character)
S3method(nc_grid_mapping_atts,data.frame)
S3method(nc_grids,NetCDF)
S3method(nc_grids,character)
S3method(nc_grids,tidync)
S3method(nc_inq,NetCDF)
S3method(nc_inq,character)
S3method(nc_meta,NetCDF)
S3method(nc_meta,character)
S3method(nc_sources,character)
S3method(nc_var,NetCDF)
S3method(nc_var,character)
S3method(nc_vars,NetCDF)
S3method(nc_vars,character)
export(nc_att)
export(nc_atts)
export(nc_axes)
export(nc_axis)
export(nc_coord_var)
export(nc_dim)
export(nc_dims)
export(nc_extended)
export(nc_gm_to_prj)
export(nc_grid_mapping_atts)
export(nc_grids)
export(nc_inq)
export(nc_meta)
export(nc_prj_to_gridmapping)
export(nc_sources)
export(nc_var)
export(nc_vars)
importFrom(RNetCDF,file.inq.nc)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(rlang,.data)
importFrom(stats,setNames)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
